<?php

App::uses('AppComponent', 'Controller/Component');
App::uses('BasketLib', 'EvBasket.Lib');

class PostStageComponent extends AppComponent {

	/**
	 * check for a process any registration data
	 *
	 * @return bool
	 */
	public function processRegister() {
		$return = true;

		if (isset($this->_controller->request->data['Form']['action']) && $this->_controller->request->data['Form']['action'] == 'register') {

			if (empty($this->_controller->UserManager)) {
				$this->_controller->UserManager = $this->_controller->loadComponent('EvCore.UserManager');
			}

			// move the array data to 'User' section, separated to prevent
			// login / register forms from both populating

			$UserModel = EvClassREgistry::init('EvCore.User');
			$result = $this->_controller->UserManager->register($UserModel);

			if ($result['result'] === true) {
				$this->_controller->Flash->success($result['data']['message']);
			} else {

				$return = false;
				$this->_controller->Flash->fail($result['data']['errors']);
			}
		}

		return $return;
	}

	/**
	 * check for a process any login data
	 *
	 * @return bool
	 */
	public function processLogin() {
		$return = true;

		if (isset($this->_controller->request->data['Form']['action']) && $this->_controller->request->data['Form']['action'] == 'login') {

			if (empty($this->_controller->UserManager)) {
				$this->_controller->UserManager = $this->_controller->loadComponent('EvCore.UserManager');
			}

			// move the array data to 'User' section, separated to prevent
			// login / register forms from both populating
			$this->_controller->request->data['User'] = $this->_controller->request->data['Login'];

			$UserModel = EvClassRegistry::init('EvCore.User');
			$result = $this->_controller->UserManager->login($UserModel);

			if ($result['result'] === true) {
				$this->_controller->Flash->success($result['data']['message']);
			} else {
				$return = false;
				$this->_controller->Flash->fail($result['data']['errors']);
			}
		}

		return $return;
	}

	/**
	 * process the delivery addresses
	 *
	 * @return 	bool
	 */
	public function processDeliveryAddress() {
		if (! CakePlugin::loaded('EvAddressBook')) {
			$this->_controller->Flash->fail('EvAddressBook is not loaded. Cannot retrieve addresses');
			return false;
		}

		$return = true;

		// check to see if we are wanting to add and select an address
		if (isset($this->_controller->request->data['Form']['action']) && $this->_controller->request->data['Form']['action'] == 'addAddress') {
			if (empty($this->_controller->AddressManager)) {
				$this->_controller->AddressManager = $this->_controller->loadComponent('EvAddressBook.AddressManager');
			}

			$AddressModel = EvClassRegistry::init('EvAddressBook.Address');
			$result = $this->_controller->AddressManager->add(
				$AddressModel,
				'EvCore.User',
				$this->_controller->Auth->user('User.id')
			);

			// if successful select the address
			if ($result['result'] === true) {
				$this->_controller->Flash->success($result['data']['message']);
				$this->_setAddress(
					'delivery',
					$result['data']['Address']['id']
				);
			} else {
				$return = false;
				$this->_controller->Flash->fail($result['data']['errors']);
			}
		}

		// check to see if we are wanting to select an existing address
		if (isset($this->_controller->request->data['Form']['action']) && $this->_controller->request->data['Form']['action'] == 'selectAddress') {
			if (empty($this->_controller->AddressManager)) {
				$this->_controller->AddressManager = $this->_controller->loadComponent('EvAddressBook.AddressManager');
			}

			$AddressModel = EvClassRegistry::init('EvAddressBook.Address');
			$result = $this->_controller->AddressManager->addressBelongsToUser(
				$AddressModel,
				$this->_controller->request->data['Address']['id'],
				$this->_controller->Auth->user('User.id')
			);

			if ($result['result'] === true) {
				$this->_setAddress(
					'delivery',
					$this->_controller->request->data['Address']['id']
				);
			} else {
				$return = false;
				$this->_controller->Flash->fail($result['data']['errors']);
			}
		}

		return $return;
	}

	/**
	 * build an order from the basket
	 *
	 * @return bool
	 */
	public function buildOrderFromBasket() {
		if (! CakePlugin::loaded('EvBasket')) {
			$this->_controller->Flash->fail('EvBasket is not loaded. Cannot process basket');
			return false;
		}

		if (! CakePlugin::loaded('EvAddressBook')) {
			$this->_controller->Flash->fail('EvAddressBook is not loaded. Cannot retrieve addresses');
			return false;
		}

		// load the address manager component
		if (empty($this->_controller->AddressManager)) {
			$this->_controller->AddressManager = $this->_controller->loadComponent('EvAddressBook.AddressManager');
		}

		// load the mbasket manager so we can get the basket
		if (empty($this->_controller->BasketManager)) {
			$this->_controller->BasketManager = $this->_controller->loadComponent('EvBasket.Basket');
		}

		// load the order manager component
		if (empty($this->_controller->OrderManager)) {
			$this->_controller->OrderManager = $this->_controller->loadComponent('EvCheckout.OrderManager');
		}

		// find the addresses to save to the order
		$addresses = $this->_findAddresses();

		// format the addresses
		App::uses('AddressBookLib', 'EvAddressBook.Lib');

		foreach ($addresses as $key => $address) {
			$addresses[$key] = AddressBookLib::display($address);
		}

		$orderId = $this->_controller->OrderManager->createFromBasket(
			$this->_controller->BasketManager->getBasket(),
			$this->_controller->Auth->user('User'),
			$addresses
		);

		debug($orderId);
		die();
		return true;
	}

	/**
	 * build a transaction from a basket
	 *
	 * @return bool
	 */
	public function buildTransactionFromBasket() {
		if (! CakePlugin::loaded('EvBasket')) {
			$this->_controller->Flash->fail('EvBasket is not loaded. Cannot process basket');
			return false;
		}

		if (! CakePlugin::loaded('EvAddressBook')) {
			$this->_controller->Flash->fail('EvAddressBook is not loaded. Cannot retrieve addresses');
			return false;
		}

		if (! CakePlugin::loaded('EvTransactions')) {
			$this->_controller->Flash->fail('EvTransactions is not loaded. Cannot setup transaction');
			return false;
		}

		// load the transactions component
		if (empty($this->_controller->TransactionsManager)) {
			$this->_controller->TransactionsManager = $this->_controller->loadComponent('EvTransactions.Transactions');
		}

		// load the address manager component
		if (empty($this->_controller->AddressManager)) {
			$this->_controller->AddressManager = $this->_controller->loadComponent('EvAddressBook.AddressManager');
		}

		// load the mbasket manager so we can get the basket
		if (empty($this->_controller->BasketManager)) {
			$this->_controller->BasketManager = $this->_controller->loadComponent('EvBasket.Basket');
		}

		// load the order manager component
		if (empty($this->_controller->OrderManager)) {
			$this->_controller->OrderManager = $this->_controller->loadComponent('EvCheckout.OrderManager');
		}

		// @todo Load addresses
		$basket = $this->_controller->BasketManager->getBasket();
		$grandTotal = $this->_controller->BasketManager->findTotalRow(Configure::read('EvBasket.labels.grandtotal'));

		$subTotal = $this->_controller->BasketManager->findTotalRow(Configure::read('EvBasket.labels.subtotal'));
		$taxTotal = $this->_controller->BasketManager->findTotalRow(Configure::read('EvBasket.labels.tax'));
		$deliveryTotal = $this->_controller->BasketManager->findTotalRow(Configure::read('EvBasket.labels.delivery'));

		$transactionTotals = array(
			'delivery' => $deliveryTotal['amount'],
			'tax' => $taxTotal['amount'],
			'subtotal' => $subTotal['amount']
		);

		$items = array();
		foreach ($basket['BasketItem'] as $BasketItem) {
			$items[] = array(
				'model' => $BasketItem['model'],
				'model_id' => $BasketItem['model_id'],
				'description' => BasketLib::getItemName($BasketItem),
				'amount' => $BasketItem['row_total'],
				'quantity' => $BasketItem['quantity'],
				'unitPrice' => $BasketItem['unit_price']
			);
		}

		// find the addresses to save to the transaction
		$addresses = $this->_findAddresses();

		$this->_controller->TransactionsManager->takePayment(
			array(
				'return' => 'http://brighton-tools.dev/success',
				'cancel' => 'http://brighton-tools.dev/fail'
			),
			array(
				'model' => 'EvBasket.Basket',
				'model' => $basket['Basket']['id']
			),
			$grandTotal['amount'],
			$items,
			array(
				'totals' => $transactionTotals,
				'delivery' => $addresses['delivery']
			)
		);

		return true;
	}

	/**
	 * set the selected address to a session
	 *
	 * @param 	string 	$type 		Type of address (billing / devliery)
	 * @param 	int 	$addressId 	Address row ID
	 * @return 	bool
	 */
	protected function _setAddress($type, $addressId) {
		return $this->_controller->Session->write(
			'EvCheckout.address.' . $type,
			$addressId
		);
	}

	/**
	 * find addresses for us to setup order with
	 *
	 * @return 	array
	 */
	protected function _findAddresses() {
		// if the addresses have been loaded and saved to the template get from there
		if (isset($this->_controller->viewVars['selectedAddresses'])) {
			$addresses = $this->_controller->viewVars['selectedAddresses'];
		} else {
			// read for the session and process each id
			$addressIds = $this->_controller->Session->read('EvCheckout.address');
			$addresses = array();

			foreach ($addressIds as $key => $addressId) {
				$addresses[$key] = $this->_controller->AddressManager->getAddress(
					EvClassRegistry::init('EvAddressBook.Address'),
					$addressIds[$key]
				);
			}
		}

		return $addresses;
	}
}
